require_relative "spec_helper"

describe Gemnigma::RotatorEncrypt do
  before :each do
    @rot_enc = Gemnigma::RotatorEncrypt.new
  end

  describe "#initialize" do
    context "with one optionally defined parameter" do
      it "should return an instance of RotatorEncrypt" do
        rot = Gemnigma::RotatorEncrypt.new(Gemnigma::Key.new)
        expect(rot).to be_an_instance_of Gemnigma::RotatorEncrypt
      end

      it "should default to 41521 if an invalid key is used for initialization" do
        rot = Gemnigma::RotatorEncrypt.new(21345)
        expect(rot.key_map).to eql([41, 15, 52, 21])
      end
    end

    context "without parameters" do
      it "should return an instance of RotatorEncrypt" do
        expect(@rot_enc).to be_an_instance_of Gemnigma::RotatorEncrypt
      end

      it "should default to 41521 as the key map" do
        expect(@rot_enc.key_map).to eql([41, 15, 52, 21])
      end
    end
  end

  describe "#char_rotation" do
    it "should be a private method" do
      expect{ @rot_enc.char_rotation }.to raise_error(NoMethodError)
    end

    it "should exist as a method of the class RotatorEncrypt" do
      expect(@rot_enc.private_methods.include? :char_rotation).to be true
    end

    it "should return an array" do
      expect(@rot_enc.send(:char_rotation).class).to eql(Array)
    end

    it "should return an array with a length of 4" do
      expect(@rot_enc.send(:char_rotation).length).to eql(4)
    end

    it "should return an array of numbers" do
      expect(@rot_enc.send(:char_rotation).all? {|num| num.is_a? Integer}).to be true
    end
  end

  describe "#map_word_to_key" do
    it "should be a private method" do
      expect{ @rot_enc.map_word_to_key }.to raise_error(NoMethodError)
    end

    it "should take one parameter" do
      expect{ @rot_enc.send(:map_word_to_key) }.to raise_error(ArgumentError)
      expect{ @rot_enc.send(:map_word_to_key, "hello", "world") }.to raise_error(ArgumentError)
    end

    it "should return an array" do
      expect(@rot_enc.send(:map_word_to_key, "Hello").class).to eql(Array)
    end

    it "should return an array of arrays" do
      expect(@rot_enc.send(:map_word_to_key, "Hello").all? {|item| item.is_a? Array}).to be true
    end

    it "should return an array of arrays containing a string and a number" do
      result = @rot_enc.send(:map_word_to_key, "Hello").map do |arr|
        arr.all? {|item| item.class == String || item.class == Fixnum}
      end
      expect(result.all?).to be true
    end

    it "should return an array length based on passed in word" do
      expect(@rot_enc.send(:map_word_to_key, "H").length).to eql(1)
      expect(@rot_enc.send(:map_word_to_key, "He").length).to eql(2)
      expect(@rot_enc.send(:map_word_to_key, "Hel").length).to eql(3)
      expect(@rot_enc.send(:map_word_to_key, "Hell").length).to eql(4)
      expect(@rot_enc.send(:map_word_to_key, "Hello").length).to eql(5)
    end
  end

  describe "#rot_encrypt" do
    it "should take one parameter" do
      expect{ @rot_enc.rot_encrypt }.to raise_error(ArgumentError)
    end

    it "should return a new string" do
      expect(@rot_enc.rot_encrypt("hello")).not_to eql("hello")
    end
  end

  describe "#get_offset" do
    it "should be a private method" do
      expect{ @rot_enc.get_offset }.to raise_error(NoMethodError)
    end

    it "should return an array" do
      expect(@rot_enc.send(:get_offset)).to be_an(Array)
    end

    it "should return an array of integers" do
      expect(@rot_enc.send(:get_offset).all? {|item| item.class == Fixnum }).to be true
    end
  end

  describe "#map_key_to_offset" do
    it "should be a private method" do
      expect{ @rot_enc.map_key_to_offset }.to raise_error(NoMethodError)
    end

    it "should return an array" do
      expect(@rot_enc.send(:map_key_to_offset)).to be_an(Array)
    end

    it "should return an array with four items" do
      expect(@rot_enc.send(:map_key_to_offset).length).to eql(4)
    end

    it "should return a nested array" do
      expect(@rot_enc.send(:map_key_to_offset).all? {|item| item.class == Array }).to be true
    end
  end

end