require_relative "spec_helper"

describe Gemnigma::Crack do
  before :each do
    @crack = Gemnigma::Crack.new
  end

  it "should be an instance of the class Crack" do
    expect(@crack).to be_an_instance_of Gemnigma::Crack
  end

  describe "#get_cmd_args" do
    it "should take no arguments" do
      expect{ @crack.get_cmd_args(2) }.to raise_error(ArgumentError)
    end

   it "should display an error message if few arguments are passed as command line arguments" do

      io = double("io", puts: nil)
      message = Gemnigma::Messages.new(io)
      message.few_args

      expect(io).to have_received(:puts).with("Not enough arguments passed in. Run Gemnigma --help for instructions on usage\nExiting...")
    end

    it "should display an error message if too many arguments are passed as command line arguments" do

      io = double("io", puts: nil)
      message = Gemnigma::Messages.new(io)
      message.too_much_args

      expect(io).to have_received(:puts).with("Too many arguments passed in. Run Gemnigma --help for instructions on usage\nExiting...")
    end
  end

  describe "#true_date?" do
    it "should be a private method" do
      expect{ @crack.true_date?(2) }.to raise_error(NoMethodError)
    end

    it "should take one compulsory argument" do
      expect{ @crack.send(:true_date?) }.to raise_error(ArgumentError)
      expect{ @crack.send(:true_date?, 123456, 90919) }.to raise_error(ArgumentError)
    end

    it "should return true for valid dates" do
      expect(@crack.send(:true_date?, 11015)).to be true
      expect(@crack.send(:true_date?, 81015)).to be true
    end

    it "should return false for invalid dates" do
      expect(@crack.send(:true_date?, 110152)).to be false
      expect(@crack.send(:true_date?, "hello")).to be false
    end
  end

  describe "#brute_force_key_map" do
    it "should be a private method" do
      expect{ @crack.brute_force_key_map }.to raise_error(NoMethodError)
    end

    it "should display an error message if an invalid date is passed in" do
      
      io = double("io", puts: nil)
      message = Gemnigma::Messages.new(io)
      message.wrong_date("hello")

      expect(io).to have_received(:puts).with("hello doesn't match the specified date format\nExiting...")
    end

    it "should throw an error if file to be read from doesn't exist" do
      expect{ @crack.send(:brute_force_key_map, "unexistent.txt", 11015) }.to raise_error(StandardError)
    end

  end

  describe "#pad_key" do
    it "should be a private method" do
      expect{ @crack.pad_key(2) }.to raise_error(NoMethodError)
    end

    it "should take one compulsory argument" do
      expect{ @crack.send(:pad_key) }.to raise_error(ArgumentError)
    end

    it "should return a padded key if the length is one" do
      expect(@crack.send(:pad_key, 0)).to eql("00")
      expect(@crack.send(:pad_key, 1)).to eql("01")
      expect(@crack.send(:pad_key, 2)).to eql("02")
      expect(@crack.send(:pad_key, 3)).to eql("03")
      expect(@crack.send(:pad_key, 4)).to eql("04")
    end

    it "should return a non-padded key if the length is greater than one" do
      expect(@crack.send(:pad_key, 10)).to eql("10")
      expect(@crack.send(:pad_key, 121)).to eql("121")
      expect(@crack.send(:pad_key, 1234)).to eql("1234")
      expect(@crack.send(:pad_key, 12345)).to eql("12345")
    end
  end

  describe "#key_gen" do
    it "should be a private method" do
      expect{ @crack.key_gen(2, 6) }.to raise_error(NoMethodError)
    end

    it "should return an array" do
      expect(@crack.send(:key_gen, 2, 6).class).to eql(Array)
    end

    it "should return an array of generated keys" do
      expect(@crack.send(:key_gen, 4, 34)).to eql([43, 34])
    end

    it "should return an empty array if key could not be generated" do
      expect(@crack.send(:key_gen, 2, 6)).to eql([])
    end
  end

  describe "#generate_cracked_key" do
    it "should be a private method" do
      expect{ @crack.generate_cracked_key }.to raise_error(NoMethodError)
    end

    it "should display a success message if file was successfully created" do
      io = double("io", puts: nil)
      message = Gemnigma::Messages.new(io)
      message.success_message("file.txt", "94704", "81015")

      expect(io).to have_received(:puts).with("\n........................................................")
      expect(io).to have_received(:puts).with("\nCreated 'file.txt' with secret key 94704 and date 81015")
    end
  end

  describe "#map_word_to_crack" do
    it "should be a private method" do
      expect{ @crack.map_word_to_crack }.to raise_error(NoMethodError)
    end

    it "should take one compulsory argument" do
      expect{ @crack.send(:map_word_to_crack) }.to raise_error(ArgumentError)
      expect{ @crack.send(:map_word_to_crack, "hello", "ade") }.to raise_error(ArgumentError)
      expect{ @crack.send(:map_word_to_crack, "hello", "ade", "one") }.to raise_error(ArgumentError)
    end
  end

  describe "#brute_crack" do
    it "should be a private method" do
      expect{ @crack.brute_crack }.to raise_error(NoMethodError)
    end

    it "should take one compulsory argument" do
      expect{ @crack.send(:brute_crack) }.to raise_error(ArgumentError)
      expect{ @crack.send(:brute_crack, 2, 3) }.to raise_error(ArgumentError)
      expect{ @crack.send(:brute_crack, 2, 3, "ade") }.to raise_error(ArgumentError)
    end
  end

  describe "#crack_file" do
    it "should be a private method" do
      expect{ @crack.crack_file("encoded.txt", "cracked.txt") }.to raise_error(NoMethodError)
    end

    it "should take two compulsory arguments" do
      expect{ @crack.send(:crack_file) }.to raise_error(ArgumentError)
      expect{ @crack.send(:crack_file, "file.txt") }.to raise_error(ArgumentError)
      expect{ @crack.send(:crack_file, "file.txt", "out.txt", "last.txt") }.to raise_error(ArgumentError)
    end
  end
end